@@ -8,32 +8,34 @@ package initcmd
 import (
 	"context"
 	"fmt"
-	"os"
 
-	"github.com/sirupsen/logrus"
 	"github.com/spf13/afero"
-	"github.com/suzuki-shunsuke/pinact/v3/pkg/controller/run"
-	"github.com/suzuki-shunsuke/pinact/v3/pkg/github"
-	"github.com/suzuki-shunsuke/urfave-cli-v3-util/log"
+	"github.com/suzuki-shunsuke/pinact/v3/pkg/cli/flag"
+	"github.com/suzuki-shunsuke/pinact/v3/pkg/controller/initcmd"
+	"github.com/suzuki-shunsuke/slog-util/slogutil"
 	"github.com/urfave/cli/v3"
 )
 
-// New creates a new init command instance with the provided logger.
-// It returns a CLI command that can be registered with the main CLI application.
-func New(logE *logrus.Entry) *cli.Command {
-	r := &runner{
-		logE: logE,
-	}
-	return r.Command()
+type Flags struct {
+	*flag.GlobalFlags
+
+	Args     []string
+	FirstArg string
 }
 
-type runner struct {
-	logE *logrus.Entry
+// New creates a new init command instance with the provided logger.
+// It returns a CLI command that can be registered with the main CLI application.
+func New(logger *slogutil.Logger, globalFlags *flag.GlobalFlags) *cli.Command {
+	r := &runner{}
+	return r.Command(logger, globalFlags)
 }
 
+type runner struct{}
+
 // Command returns the CLI command definition for the init subcommand.
 // It defines the command name, usage, description, and action handler.
-func (r *runner) Command() *cli.Command {
+func (r *runner) Command(logger *slogutil.Logger, globalFlags *flag.GlobalFlags) *cli.Command {
+	flags := &Flags{GlobalFlags: globalFlags}
 	return &cli.Command{
 		Name:  "init",
 		Usage: "Create .pinact.yaml if it doesn't exist",
@@ -47,7 +49,11 @@ e.g.
 
 $ pinact init .github/pinact.yaml
 `,
-		Action: r.action,
+		Action: func(ctx context.Context, cmd *cli.Command) error {
+			flags.Args = cmd.Args().Slice()
+			flags.FirstArg = cmd.Args().First()
+			return r.action(ctx, logger, flags)
+		},
 	}
 }
 
@@ -55,35 +61,17 @@ $ pinact init .github/pinact.yaml
 // It creates a default .pinact.yaml configuration file in the specified location.
 // The function sets up the necessary controllers and services, determines the output
 // path for the configuration file, and delegates to the controller's Init method.
-func (r *runner) action(ctx context.Context, c *cli.Command) error {
-	pwd, err := os.Getwd()
-	if err != nil {
-		return fmt.Errorf("get the current directory: %w", err)
+func (r *runner) action(_ context.Context, logger *slogutil.Logger, flags *Flags) error {
+	if err := logger.SetLevel(flags.LogLevel); err != nil {
+		return fmt.Errorf("set log level: %w", err)
 	}
-	gh := github.New(ctx, r.logE)
-	ctrl := run.New(&run.RepositoriesServiceImpl{
-		Tags:                map[string]*run.ListTagsResult{},
-		Releases:            map[string]*run.ListReleasesResult{},
-		Commits:             map[string]*run.GetCommitSHA1Result{},
-		RepositoriesService: gh.Repositories,
-	}, gh.PullRequests, afero.NewOsFs(), nil, nil, &run.ParamRun{
-		WorkflowFilePaths: c.Args().Slice(),
-		ConfigFilePath:    c.String("config"),
-		PWD:               pwd,
-		IsVerify:          c.Bool("verify"),
-		Check:             c.Bool("check"),
-		Update:            c.Bool("update"),
-	})
-
-	if err := log.Set(r.logE, c.String("log-level"), "auto"); err != nil {
-		return fmt.Errorf("configure logger: %w", err)
-	}
-	configFilePath := c.Args().First()
+	configFilePath := flags.FirstArg
 	if configFilePath == "" {
-		configFilePath = c.String("config")
+		configFilePath = flags.Config
 	}
 	if configFilePath == "" {
 		configFilePath = ".pinact.yaml"
 	}
+	ctrl := initcmd.New(afero.NewOsFs())
 	return ctrl.Init(configFilePath) //nolint:wrapcheck
 }
