@@ -8,11 +8,12 @@ package cli
 import (
 	"context"
 
-	"github.com/sirupsen/logrus"
+	"github.com/suzuki-shunsuke/pinact/v3/pkg/cli/flag"
 	"github.com/suzuki-shunsuke/pinact/v3/pkg/cli/initcmd"
 	"github.com/suzuki-shunsuke/pinact/v3/pkg/cli/migrate"
 	"github.com/suzuki-shunsuke/pinact/v3/pkg/cli/run"
 	"github.com/suzuki-shunsuke/pinact/v3/pkg/cli/token"
+	"github.com/suzuki-shunsuke/slog-util/slogutil"
 	"github.com/suzuki-shunsuke/urfave-cli-v3-util/urfave"
 	"github.com/urfave/cli/v3"
 )
@@ -20,38 +21,17 @@ import (
 // Run creates and executes the main pinact CLI application.
 // It configures the command structure with global flags and subcommands,
 // then runs the CLI with the provided arguments.
-//
-// Parameters:
-//   - ctx: context for cancellation and timeout control
-//   - logE: logrus entry for structured logging
-//   - ldFlags: linker flags containing build information
-//   - args: command line arguments to parse and execute
-//
-// Returns an error if command parsing or execution fails.
-func Run(ctx context.Context, logE *logrus.Entry, ldFlags *urfave.LDFlags, args ...string) error {
-	return urfave.Command(logE, ldFlags, &cli.Command{ //nolint:wrapcheck
+func Run(ctx context.Context, logger *slogutil.Logger, env *urfave.Env) error {
+	globalFlags := &flag.GlobalFlags{}
+	return urfave.Command(env, &cli.Command{ //nolint:wrapcheck
 		Name:  "pinact",
 		Usage: "Pin GitHub Actions versions. https://github.com/suzuki-shunsuke/pinact",
-		Flags: []cli.Flag{
-			&cli.StringFlag{
-				Name:    "log-level",
-				Usage:   "log level",
-				Sources: cli.EnvVars("PINACT_LOG_LEVEL"),
-			},
-			&cli.StringFlag{
-				Name: "config",
-				Aliases: []string{
-					"c",
-				},
-				Usage:   "configuration file path",
-				Sources: cli.EnvVars("PINACT_CONFIG"),
-			},
-		},
+		Flags: globalFlags.Flags(),
 		Commands: []*cli.Command{
-			initcmd.New(logE),
-			run.New(logE),
-			migrate.New(logE),
-			token.New(logE),
+			initcmd.New(logger, globalFlags),
+			run.New(logger, globalFlags, env),
+			migrate.New(logger, globalFlags),
+			token.New(logger),
 		},
-	}).Run(ctx, args)
+	}).Run(ctx, env.Args)
 }
