@@ -4,13 +4,13 @@ import (
 	"context"
 	"errors"
 	"fmt"
+	"log/slog"
 	"regexp"
 	"strings"
 
 	"github.com/hashicorp/go-version"
-	"github.com/sirupsen/logrus"
-	"github.com/suzuki-shunsuke/logrus-error/logerr"
 	"github.com/suzuki-shunsuke/pinact/v3/pkg/github"
+	"github.com/suzuki-shunsuke/slog-error/slogerr"
 )
 
 var (
@@ -45,11 +45,6 @@ const (
 // getVersionType determines the type of version string.
 // It analyzes the version format to classify it as semantic version,
 // short semantic version, full commit SHA, empty, or other.
-//
-// Parameters:
-//   - v: version string to analyze
-//
-// Returns the VersionType classification.
 func getVersionType(v string) VersionType {
 	if v == "" {
 		return Empty
@@ -69,11 +64,6 @@ func getVersionType(v string) VersionType {
 // parseAction extracts action information from a YAML line.
 // It uses regular expressions to parse 'uses' statements and extract
 // action name, version, comments, and formatting details.
-//
-// Parameters:
-//   - line: YAML line containing a 'uses' statement
-//
-// Returns an Action struct with parsed information, or nil if no match.
 func parseAction(line string) *Action {
 	matches := usesPattern.FindStringSubmatch(line)
 	if matches == nil {
@@ -94,17 +84,11 @@ var ErrCantPinned = errors.New("action can't be pinned")
 
 // ignoreAction checks if an action should be ignored based on configuration.
 // It evaluates the action against all ignore rules in the configuration.
-//
-// Parameters:
-//   - logE: logrus entry for structured logging
-//   - action: action to check against ignore rules
-//
-// Returns true if the action should be ignored, false otherwise.
-func (c *Controller) ignoreAction(logE *logrus.Entry, action *Action) bool {
+func (c *Controller) ignoreAction(logger *slog.Logger, action *Action) bool {
 	for _, ignoreAction := range c.cfg.IgnoreActions {
 		f, err := ignoreAction.Match(action.Name, action.Version, c.cfg.Version)
 		if err != nil {
-			logerr.WithError(logE, err).Warn("match the action")
+			slogerr.WithError(logger, err).Warn("match the action")
 			continue
 		}
 		if f {
@@ -116,11 +100,6 @@ func (c *Controller) ignoreAction(logE *logrus.Entry, action *Action) bool {
 
 // excludeAction checks if an action should be excluded based on exclude patterns.
 // It tests the action name against all configured exclude regular expressions.
-//
-// Parameters:
-//   - actionName: name of the action to check
-//
-// Returns true if the action matches any exclude pattern, false otherwise.
 func (c *Controller) excludeAction(actionName string) bool {
 	for _, exclude := range c.param.Excludes {
 		if exclude.MatchString(actionName) {
@@ -133,11 +112,6 @@ func (c *Controller) excludeAction(actionName string) bool {
 // excludeByIncludes checks if an action should be excluded due to include patterns.
 // When include patterns are specified, only actions matching include patterns
 // are processed, so this returns true if the action doesn't match any include pattern.
-//
-// Parameters:
-//   - actionName: name of the action to check
-//
-// Returns true if includes are specified and action doesn't match any, false otherwise.
 func (c *Controller) excludeByIncludes(actionName string) bool {
 	if len(c.param.Includes) == 0 {
 		return false
@@ -153,66 +127,69 @@ func (c *Controller) excludeByIncludes(actionName string) bool {
 // parseLine processes a single line from a workflow file.
 // It parses the line for action usage, applies filtering rules, and determines
 // what modifications (if any) should be made based on the operation mode.
-//
-// Parameters:
-//   - ctx: context for cancellation and timeout control
-//   - logE: logrus entry for structured logging
-//   - line: workflow file line to process
-//
-// Returns the modified line content and any error encountered.
-func (c *Controller) parseLine(ctx context.Context, logE *logrus.Entry, line string) (s string, e error) { //nolint:cyclop
+func (c *Controller) parseLine(ctx context.Context, logger *slog.Logger, line string) (s string, e error) {
+	attrs := slogerr.NewAttrs(2) //nolint:mnd
 	defer func() {
-		e = logerr.WithFields(e, logE.Data)
+		e = attrs.With(e)
 	}()
 	action := parseAction(line)
 	if action == nil {
-		// Ignore a line if the line doesn't use an action.
-		logE.Debug("unmatch")
+		logger.Debug("unmatch")
 		return "", nil
 	}
 
-	logE = logE.WithField("action", action.Name+"@"+action.Version)
+	logger = attrs.Add(logger, "action", action.Name+"@"+action.Version)
 
-	if c.ignoreAction(logE, action) {
-		logE.Debug("ignore the action")
+	if c.shouldSkipAction(logger, action) {
 		return "", nil
 	}
+
+	if !c.parseActionName(action) {
+		logger.Debug("ignore line")
+		return "", nil
+	}
+
+	return c.processVersionComment(ctx, logger, action, attrs)
+}
+
+// shouldSkipAction checks if an action should be skipped based on filtering rules.
+func (c *Controller) shouldSkipAction(logger *slog.Logger, action *Action) bool {
+	if c.ignoreAction(logger, action) {
+		logger.Debug("ignore the action")
+		return true
+	}
 	if c.excludeAction(action.Name) {
-		logE.Debug("exclude the action")
-		return "", nil
+		logger.Debug("exclude the action")
+		return true
 	}
 	if c.excludeByIncludes(action.Name) {
-		logE.Debug("exclude the action")
-		return "", nil
-	}
-
-	if c.param.Check && !c.param.Diff && !c.param.Fix {
-		if fullCommitSHAPattern.MatchString(action.Version) {
-			return "", nil
-		}
-		return "", ErrActionNotPinned
-	}
-
-	if f := c.parseActionName(action); !f {
-		logE.Debug("ignore line")
-		return "", nil
+		logger.Debug("exclude the action")
+		return true
 	}
+	return false
+}
 
+// processVersionComment processes the action based on its version comment type.
+func (c *Controller) processVersionComment(ctx context.Context, logger *slog.Logger, action *Action, attrs *slogerr.Attrs) (string, error) {
 	switch getVersionType(action.VersionComment) {
 	case Empty:
-		return c.parseNoTagLine(ctx, logE, action)
+		// @xxx
+		// Note that comments like "hoge" are treated as Empty
+		return c.parseNoTagLine(ctx, logger, action)
 	case Semver:
-		// @xxx # v3.0.0
-		return c.parseSemverTagLine(ctx, logE, action)
+		// @xxx # v1.0.0
+		return c.parseSemverTagLine(ctx, logger, action)
 	case Shortsemver:
-		// @xxx # v3
-		// @<full commit hash> # v3
-		logE = logE.WithField("version_annotation", action.VersionComment)
-		return c.parseShortSemverTagLine(ctx, logE, action)
+		// @xxx # v1
+		logger = attrs.Add(logger, "version_annotation", action.VersionComment)
+		return c.parseShortSemverTagLine(ctx, logger, action)
 	default:
+		// @xxx # hoge
 		if getVersionType(action.Version) == FullCommitSHA {
+			// @<full commit sha> # hoge
 			return "", nil
 		}
+		// @<not full commit sha> # hoge
 		return "", ErrCantPinned
 	}
 }
@@ -220,14 +197,7 @@ func (c *Controller) parseLine(ctx context.Context, logE *logrus.Entry, line str
 // parseNoTagLine processes actions without version comments.
 // It handles pinning actions that don't have version annotations,
 // either by updating to latest version or converting tags to commit SHAs.
-//
-// Parameters:
-//   - ctx: context for cancellation and timeout control
-//   - logE: logrus entry for structured logging
-//   - action: parsed action information
-//
-// Returns the modified line content and any error encountered.
-func (c *Controller) parseNoTagLine(ctx context.Context, logE *logrus.Entry, action *Action) (string, error) { //nolint:cyclop
+func (c *Controller) parseNoTagLine(ctx context.Context, logger *slog.Logger, action *Action) (string, error) {
 	typ := getVersionType(action.Version)
 	switch typ {
 	case Shortsemver, Semver:
@@ -236,30 +206,37 @@ func (c *Controller) parseNoTagLine(ctx context.Context, logE *logrus.Entry, act
 	default:
 		return "", ErrCantPinned
 	}
-	// @xxx
+	// @v1, @v1.0.0
 	if c.param.Update {
-		// get the latest version
-		lv, err := c.getLatestVersion(ctx, logE, action.RepoOwner, action.RepoName, action.Version)
-		if err != nil {
-			return "", fmt.Errorf("get the latest version: %w", err)
-		}
-		sha, _, err := c.repositoriesService.GetCommitSHA1(ctx, action.RepoOwner, action.RepoName, lv, "")
-		if err != nil {
-			return "", fmt.Errorf("get a reference: %w", err)
-		}
-		return patchLine(action, sha, lv), nil
+		return c.updateToLatestVersion(ctx, logger, action)
 	}
+	return c.pinCurrentVersion(ctx, logger, action, typ)
+}
 
+// updateToLatestVersion updates an action to its latest version.
+func (c *Controller) updateToLatestVersion(ctx context.Context, logger *slog.Logger, action *Action) (string, error) {
+	lv, err := c.getLatestVersion(ctx, logger, action.RepoOwner, action.RepoName, action.Version)
+	if err != nil {
+		return "", fmt.Errorf("get the latest version: %w", err)
+	}
+	sha, _, err := c.repositoriesService.GetCommitSHA1(ctx, logger, action.RepoOwner, action.RepoName, lv, "")
+	if err != nil {
+		return "", fmt.Errorf("get a reference: %w", err)
+	}
+	return patchLine(action, sha, lv), nil
+}
+
+// pinCurrentVersion pins the current version to a commit SHA.
+func (c *Controller) pinCurrentVersion(ctx context.Context, logger *slog.Logger, action *Action, typ VersionType) (string, error) {
 	// Get commit hash from tag
 	// https://docs.github.com/en/rest/git/refs?apiVersion=2022-11-28#get-a-reference
-	// > The :ref in the URL must be formatted as heads/<branch name> for branches and tags/<tag name> for tags. If the :ref doesn't match an existing ref, a 404 is returned.
-	sha, _, err := c.repositoriesService.GetCommitSHA1(ctx, action.RepoOwner, action.RepoName, action.Version, "")
+	sha, _, err := c.repositoriesService.GetCommitSHA1(ctx, logger, action.RepoOwner, action.RepoName, action.Version, "")
 	if err != nil {
 		return "", fmt.Errorf("get a reference: %w", err)
 	}
 	longVersion := action.Version
 	if typ == Shortsemver {
-		v, err := c.getLongVersionFromSHA(ctx, action, sha)
+		v, err := c.getLongVersionFromSHA(ctx, logger, action, sha)
 		if err != nil {
 			return "", err
 		}
@@ -267,19 +244,12 @@ func (c *Controller) parseNoTagLine(ctx context.Context, logE *logrus.Entry, act
 			longVersion = v
 		}
 	}
-	// @yyy # longVersion
 	return patchLine(action, sha, longVersion), nil
 }
 
 // compareVersion compares two version strings.
 // It attempts semantic version comparison first, falling back to
 // string comparison if semantic parsing fails.
-//
-// Parameters:
-//   - currentVersion: current version string
-//   - newVersion: new version string to compare
-//
-// Returns true if newVersion is greater than currentVersion.
 func compareVersion(currentVersion, newVersion string) bool {
 	cv, err := version.NewVersion(currentVersion)
 	if err != nil {
@@ -295,18 +265,11 @@ func compareVersion(currentVersion, newVersion string) bool {
 // parseSemverTagLine processes actions with semantic version comments.
 // It handles updating semantic versions to latest and verifying that
 // commit SHAs match their corresponding version tags.
-//
-// Parameters:
-//   - ctx: context for cancellation and timeout control
-//   - logE: logrus entry for structured logging
-//   - action: parsed action information
-//
-// Returns the modified line content and any error encountered.
-func (c *Controller) parseSemverTagLine(ctx context.Context, logE *logrus.Entry, action *Action) (string, error) {
+func (c *Controller) parseSemverTagLine(ctx context.Context, logger *slog.Logger, action *Action) (string, error) {
 	// @xxx # v3.0.0
 	if c.param.Update { //nolint:nestif
 		// get the latest version
-		lv, err := c.getLatestVersion(ctx, logE, action.RepoOwner, action.RepoName, action.VersionComment)
+		lv, err := c.getLatestVersion(ctx, logger, action.RepoOwner, action.RepoName, action.VersionComment)
 		if err != nil {
 			return "", fmt.Errorf("get the latest version: %w", err)
 		}
@@ -314,14 +277,14 @@ func (c *Controller) parseSemverTagLine(ctx context.Context, logE *logrus.Entry,
 			return "", nil
 		}
 		if !compareVersion(action.VersionComment, lv) {
-			logE.WithFields(logrus.Fields{
-				"current_version": action.VersionComment,
-				"new_version":     lv,
-			}).Warn("skip updating because the current version is newer than the new version")
+			logger.Warn("skip updating because the current version is newer than the new version",
+				"current_version", action.VersionComment,
+				"new_version", lv,
+			)
 			return "", nil
 		}
 		if action.VersionComment != lv {
-			sha, _, err := c.repositoriesService.GetCommitSHA1(ctx, action.RepoOwner, action.RepoName, lv, "")
+			sha, _, err := c.repositoriesService.GetCommitSHA1(ctx, logger, action.RepoOwner, action.RepoName, lv, "")
 			if err != nil {
 				return "", fmt.Errorf("get the latest version: %w", err)
 			}
@@ -337,7 +300,7 @@ func (c *Controller) parseSemverTagLine(ctx context.Context, logE *logrus.Entry,
 	if FullCommitSHA != getVersionType(action.Version) {
 		return "", nil
 	}
-	if err := c.verify(ctx, action); err != nil {
+	if err := c.verify(ctx, logger, action); err != nil {
 		return "", fmt.Errorf("verify the version annotation: %w", err)
 	}
 	return "", nil
@@ -346,37 +309,30 @@ func (c *Controller) parseSemverTagLine(ctx context.Context, logE *logrus.Entry,
 // parseShortSemverTagLine processes actions with short semantic version comments.
 // It handles expanding short versions (like v3) to full versions (like v3.1.0)
 // and updating to latest versions when requested.
-//
-// Parameters:
-//   - ctx: context for cancellation and timeout control
-//   - logE: logrus entry for structured logging
-//   - action: parsed action information
-//
-// Returns the modified line content and any error encountered.
-func (c *Controller) parseShortSemverTagLine(ctx context.Context, logE *logrus.Entry, action *Action) (string, error) {
+func (c *Controller) parseShortSemverTagLine(ctx context.Context, logger *slog.Logger, action *Action) (string, error) {
 	// @xxx # v3
 	// @<full commit hash> # v3
 	if FullCommitSHA != getVersionType(action.Version) {
 		return "", ErrCantPinned
 	}
 	if c.param.Update {
-		lv, err := c.getLatestVersion(ctx, logE, action.RepoOwner, action.RepoName, action.VersionComment)
+		lv, err := c.getLatestVersion(ctx, logger, action.RepoOwner, action.RepoName, action.VersionComment)
 		if err != nil {
 			return "", fmt.Errorf("get the latest version: %w", err)
 		}
-		sha, _, err := c.repositoriesService.GetCommitSHA1(ctx, action.RepoOwner, action.RepoName, lv, "")
+		sha, _, err := c.repositoriesService.GetCommitSHA1(ctx, logger, action.RepoOwner, action.RepoName, lv, "")
 		if err != nil {
 			return "", fmt.Errorf("get the latest version: %w", err)
 		}
 		return patchLine(action, sha, lv), nil
 	}
 	// replace Shortsemer to Semver
-	longVersion, err := c.getLongVersionFromSHA(ctx, action, action.Version)
+	longVersion, err := c.getLongVersionFromSHA(ctx, logger, action, action.Version)
 	if err != nil {
 		return "", err
 	}
 	if longVersion == "" {
-		logE.Debug("a long tag whose SHA is same as SHA of the version annotation isn't found")
+		logger.Debug("a long tag whose SHA is same as SHA of the version annotation isn't found")
 		return "", nil
 	}
 	return patchLine(action, action.Version, longVersion), nil
@@ -385,13 +341,6 @@ func (c *Controller) parseShortSemverTagLine(ctx context.Context, logE *logrus.E
 // patchLine reconstructs a workflow line with updated version and tag.
 // It combines the action information with new version and tag to create
 // the updated line with proper formatting and comments.
-//
-// Parameters:
-//   - action: parsed action information
-//   - version: new version (commit SHA or tag)
-//   - tag: new tag for version comment
-//
-// Returns the reconstructed line string.
 func patchLine(action *Action, version, tag string) string {
 	sep := action.VersionCommentSeparator
 	if sep == "" {
@@ -403,20 +352,13 @@ func patchLine(action *Action, version, tag string) string {
 // getLongVersionFromSHA finds the full semantic version tag for a commit SHA.
 // It searches through repository tags to find a tag that points to the given
 // commit and matches the version comment prefix.
-//
-// Parameters:
-//   - ctx: context for cancellation and timeout control
-//   - action: parsed action information
-//   - sha: commit SHA to search for
-//
-// Returns the matching full version tag or empty string if not found.
-func (c *Controller) getLongVersionFromSHA(ctx context.Context, action *Action, sha string) (string, error) {
+func (c *Controller) getLongVersionFromSHA(ctx context.Context, logger *slog.Logger, action *Action, sha string) (string, error) {
 	opts := &github.ListOptions{
 		PerPage: 100, //nolint:mnd
 	}
 	// Get long tag from commit hash
 	for range 10 {
-		tags, resp, err := c.repositoriesService.ListTags(ctx, action.RepoOwner, action.RepoName, opts)
+		tags, resp, err := c.repositoriesService.ListTags(ctx, logger, action.RepoOwner, action.RepoName, opts)
 		if err != nil {
 			return "", fmt.Errorf("list tags: %w", err)
 		}
@@ -449,11 +391,6 @@ func (c *Controller) getLongVersionFromSHA(ctx context.Context, action *Action,
 // parseActionName extracts repository owner and name from action name.
 // It parses the action name to extract the repository owner and name
 // components, which are needed for GitHub API calls.
-//
-// Parameters:
-//   - action: action to parse (modifies RepoOwner and RepoName fields)
-//
-// Returns true if parsing successful, false if action name is invalid.
 func (c *Controller) parseActionName(action *Action) bool {
 	a := strings.Split(action.Name, "/")
 	if len(a) == 1 {
@@ -468,25 +405,19 @@ func (c *Controller) parseActionName(action *Action) bool {
 // verify checks that an action's version SHA matches its version comment.
 // It validates that the commit SHA in the action version matches the
 // commit SHA of the version specified in the comment.
-//
-// Parameters:
-//   - ctx: context for cancellation and timeout control
-//   - action: parsed action information to verify
-//
-// Returns an error if verification fails, nil if successful.
-func (c *Controller) verify(ctx context.Context, action *Action) error {
-	sha, _, err := c.repositoriesService.GetCommitSHA1(ctx, action.RepoOwner, action.RepoName, action.VersionComment, "")
+func (c *Controller) verify(ctx context.Context, logger *slog.Logger, action *Action) error {
+	sha, _, err := c.repositoriesService.GetCommitSHA1(ctx, logger, action.RepoOwner, action.RepoName, action.VersionComment, "")
 	if err != nil {
 		return fmt.Errorf("get a commit hash: %w", err)
 	}
 	if action.Version == sha {
 		return nil
 	}
-	return logerr.WithFields(errors.New("action_version must be equal to commit_hash_of_version_annotation"), logrus.Fields{ //nolint:wrapcheck
-		"action":                            action.Name,
-		"action_version":                    action.Version,
-		"version_annotation":                action.VersionComment,
-		"commit_hash_of_version_annotation": sha,
-		"help_docs":                         "https://github.com/suzuki-shunsuke/pinact/blob/main/docs/codes/001.md",
-	})
+	return slogerr.With(errors.New("action_version must be equal to commit_hash_of_version_annotation"), //nolint:wrapcheck
+		"action", action.Name,
+		"action_version", action.Version,
+		"version_annotation", action.VersionComment,
+		"commit_hash_of_version_annotation", sha,
+		"help_docs", "https://github.com/suzuki-shunsuke/pinact/blob/main/docs/codes/001.md",
+	)
 }
