@@ -9,11 +9,10 @@ package github
 
 import (
 	"context"
+	"log/slog"
 	"net/http"
-	"os"
 
-	"github.com/google/go-github/v79/github"
-	"github.com/sirupsen/logrus"
+	"github.com/google/go-github/v80/github"
 	"github.com/suzuki-shunsuke/urfave-cli-v3-util/keyring/ghtoken"
 	"golang.org/x/oauth2"
 )
@@ -22,70 +21,38 @@ type (
 	ListOptions        = github.ListOptions
 	Reference          = github.Reference
 	Response           = github.Response
+	Repository         = github.Repository
 	RepositoryTag      = github.RepositoryTag
 	RepositoryRelease  = github.RepositoryRelease
 	Client             = github.Client
 	GitObject          = github.GitObject
 	Commit             = github.Commit
+	CommitAuthor       = github.CommitAuthor
+	Timestamp          = github.Timestamp
 	PullRequestComment = github.PullRequestComment
 )
 
 // New creates a new GitHub API client with authentication.
 // It configures the client with appropriate HTTP client based on available
 // authentication methods (environment token or keyring).
-//
-// Parameters:
-//   - ctx: context for OAuth2 token source
-//   - logE: logrus entry for structured logging
-//
-// Returns a configured GitHub API client.
-func New(ctx context.Context, logE *logrus.Entry) *Client {
-	return github.NewClient(getHTTPClientForGitHub(ctx, logE, getGitHubToken()))
+func New(ctx context.Context, logger *slog.Logger, token string, keyringEnabled bool) *Client {
+	return github.NewClient(getHTTPClientForGitHub(ctx, logger, token, keyringEnabled))
 }
 
 // Ptr returns a pointer to the provided value.
 // This is a convenience function that delegates to github.Ptr for
 // creating pointers to values, commonly needed for GitHub API structs.
-//
-// Parameters:
-//   - v: value to get a pointer to
-//
-// Returns a pointer to the value.
 func Ptr[T any](v T) *T {
 	return github.Ptr(v)
 }
 
-// getGitHubToken retrieves the GitHub token from environment variables.
-// It reads the GITHUB_TOKEN environment variable for authentication.
-//
-// Returns the GitHub token string or empty string if not set.
-func getGitHubToken() string {
-	return os.Getenv("GITHUB_TOKEN")
-}
-
-// checkKeyringEnabled checks if keyring authentication is enabled.
-// It examines the PINACT_KEYRING_ENABLED environment variable to determine
-// if OS keyring should be used for token storage and retrieval.
-//
-// Returns true if keyring is enabled, false otherwise.
-func checkKeyringEnabled() bool {
-	return os.Getenv("PINACT_KEYRING_ENABLED") == "true"
-}
-
 // getHTTPClientForGitHub creates an HTTP client configured for GitHub API access.
 // It handles authentication using environment token, keyring, or falls back
 // to unauthenticated access. The client is configured with OAuth2 for authenticated requests.
-//
-// Parameters:
-//   - ctx: context for OAuth2 token source
-//   - logE: logrus entry for structured logging
-//   - token: GitHub token for authentication (empty string for alternative auth)
-//
-// Returns an HTTP client configured for GitHub API access.
-func getHTTPClientForGitHub(ctx context.Context, logE *logrus.Entry, token string) *http.Client {
+func getHTTPClientForGitHub(ctx context.Context, logger *slog.Logger, token string, keyringEnabled bool) *http.Client {
 	if token == "" {
-		if checkKeyringEnabled() {
-			return oauth2.NewClient(ctx, ghtoken.NewTokenSource(logE, KeyService))
+		if keyringEnabled {
+			return oauth2.NewClient(ctx, ghtoken.NewTokenSource(logger, KeyService))
 		}
 		return http.DefaultClient
 	}
@@ -93,3 +60,21 @@ func getHTTPClientForGitHub(ctx context.Context, logE *logrus.Entry, token strin
 		&oauth2.Token{AccessToken: token},
 	))
 }
+
+// NewWithBaseURL creates a new GitHub API client with a custom base URL.
+// This is used for GitHub Enterprise Server instances.
+func NewWithBaseURL(ctx context.Context, baseURL, token string) (*Client, error) {
+	httpClient := getHTTPClientForGitHubWithToken(ctx, token)
+	return github.NewClient(httpClient).WithEnterpriseURLs(baseURL, baseURL) //nolint:wrapcheck
+}
+
+// getHTTPClientForGitHubWithToken creates an HTTP client with a specific token.
+// Unlike getHTTPClientForGitHub, this does not fall back to keyring.
+func getHTTPClientForGitHubWithToken(ctx context.Context, token string) *http.Client {
+	if token == "" {
+		return http.DefaultClient
+	}
+	return oauth2.NewClient(ctx, oauth2.StaticTokenSource(
+		&oauth2.Token{AccessToken: token},
+	))
+}
